"use strict";
/* eslint-disable @typescript-eslint/no-explicit-any */
/* eslint-disable no-console */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const pg_format_1 = __importDefault(require("pg-format"));
const query_service_1 = __importDefault(require("./query.service"));
const constants_1 = require("../../../utils/constants");
class PostgresqlClientService {
    buildFilterClause(filters, logicOperator) {
        const filterClauses = filters.map((filter) => {
            if ('attribute' in filter) {
                return `${pg_format_1.default.ident(filter.attribute)} ${filter.operator} ${pg_format_1.default.literal(filter.value)}`;
            }
            else {
                return `(${this.buildFilterClause(filter.filters, filter.logicOperator)})`;
            }
        });
        const operator = logicOperator !== null && logicOperator !== void 0 ? logicOperator : constants_1.FILTER_LOGIC_OPERATORS.AND;
        return filterClauses.join(` ${operator} `);
    }
    buildQueryFilters(filterExpressions) {
        if (!(filterExpressions === null || filterExpressions === void 0 ? void 0 : filterExpressions.filters))
            return;
        const filterClause = this.buildFilterClause(filterExpressions === null || filterExpressions === void 0 ? void 0 : filterExpressions.filters, filterExpressions === null || filterExpressions === void 0 ? void 0 : filterExpressions.logicOperator);
        return filterClause;
    }
    formatOrderByItem(item) {
        if (typeof item === 'string') {
            return pg_format_1.default.ident(item);
        }
        else if (typeof item === 'object' && item !== null) {
            const [column, orientation] = Object.entries(item)[0];
            return `${pg_format_1.default.ident(column)} ${orientation.toUpperCase()}`;
        }
        else {
            throw new Error();
        }
    }
    buildOrderByClause(orderBy) {
        try {
            return orderBy.map(this.formatOrderByItem).join(', ');
        }
        catch (error) {
            return '';
        }
    }
    async createCommand(tableName, data) {
        const keys = Object.keys(data);
        const values = Object.values(data);
        const query = (0, pg_format_1.default)(`INSERT INTO %I (%I) VALUES (%L) RETURNING *;`, tableName, keys, values);
        return query_service_1.default.executeQuery(query);
    }
    addFiltersToQuery(query, filters) {
        if (filters)
            query += ` WHERE ${this.buildQueryFilters(filters)}`;
        return query;
    }
    addOrderByToQuery(query, orderBy) {
        if (orderBy)
            query += ` ORDER BY ${this.buildOrderByClause(orderBy)}`;
        return query;
    }
    addPaginationToQuery(query, pagination) {
        if (pagination) {
            const { limit, from } = pagination;
            if (limit)
                query += ` LIMIT ${limit}`;
            if (from)
                query += ` OFFSET ${from}`;
        }
        return query;
    }
    getSelectClause(fields) {
        if (!(fields === null || fields === void 0 ? void 0 : fields.length))
            return '*';
        return fields.join(', ');
    }
    async findCommand(tableName, options) {
        let query = `SELECT ${this.getSelectClause(options === null || options === void 0 ? void 0 : options.fields)} FROM ${pg_format_1.default.ident(tableName)}`;
        query = this.addFiltersToQuery(query, options === null || options === void 0 ? void 0 : options.filters);
        query = this.addOrderByToQuery(query, options === null || options === void 0 ? void 0 : options.orderBy);
        query = this.addPaginationToQuery(query, options === null || options === void 0 ? void 0 : options.pagination);
        return query_service_1.default.executeQuery(query);
    }
    async updateCommand(tableName, filters, data) {
        let query = `UPDATE ${pg_format_1.default.ident(tableName)} SET`;
        const updateClauses = Object.entries(data).map(([key, value]) => {
            return `${pg_format_1.default.ident(key)} = ${pg_format_1.default.literal(value)}`;
        });
        query += ` ${updateClauses.join(', ')}`;
        query += ' WHERE ';
        query += this.buildFilterClause(filters.filters, filters.logicOperator);
        return query_service_1.default.executeQuery(query);
    }
    async deleteCommand(tableName, filters) {
        let query = `DELETE FROM ${pg_format_1.default.ident(tableName)}`;
        if (filters) {
            query += ' WHERE ';
            query += this.buildFilterClause(filters.filters, filters.logicOperator);
        }
        return query_service_1.default.executeQuery(query);
    }
    executeQuery(queryText, values) {
        return query_service_1.default.executeQuery(queryText, values);
    }
}
exports.default = PostgresqlClientService;
//# sourceMappingURL=postgreSqlClient.service.js.map